"""
Initialize the app
"""

# Django
from django.utils.translation import gettext_lazy as _

__version__ = "2.7.2"
__title__ = _("Fleet Finder")
__verbose_name__ = "Fleet Finder for Alliance Auth"

__package_name__ = "aa-fleetfinder"
__app_name__ = "fleetfinder"
__app_name_verbose__ = "AA Fleet Finder"
__app_name_useragent__ = "AA-Fleet-Finder"

__github_url__ = f"https://github.com/ppfeufer/{__package_name__}"
