# Generated by Django 4.0.10 on 2023-09-22 18:31

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("eveonline", "0017_alliance_and_corp_names_are_not_unique"),
        ("groupmanagement", "0019_adding_restricted_to_groups"),
        ("fleetfinder", "0002_esi_error_handling_and_verbose_names"),
    ]

    operations = [
        migrations.AlterField(
            model_name="fleet",
            name="fleet_commander",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="fleetfinder_fleet_commander",
                to="eveonline.evecharacter",
                verbose_name="Fleet commander",
            ),
        ),
        migrations.AlterField(
            model_name="fleet",
            name="groups",
            field=models.ManyToManyField(
                help_text="Only selected groups will have access to the fleet. If no groups are selected the fleet will be available to all who have access to this module.",
                related_name="fleetfinder_restricted_groups",
                to="groupmanagement.authgroup",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="fleet",
            name="name",
            field=models.CharField(
                default="", max_length=50, verbose_name="Fleet name"
            ),
        ),
    ]
